// Code generated by smithy-go-codegen DO NOT EDIT.

package codecommit

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/aws/signer/v4"
	"github.com/aws/aws-sdk-go-v2/service/codecommit/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Returns information about comments made on the comparison between two commits.
// Reaction counts might include numbers from user identities who were deleted
// after the reaction was made. For a count of reactions from active identities,
// use GetCommentReactions.
func (c *Client) GetCommentsForComparedCommit(ctx context.Context, params *GetCommentsForComparedCommitInput, optFns ...func(*Options)) (*GetCommentsForComparedCommitOutput, error) {
	if params == nil {
		params = &GetCommentsForComparedCommitInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "GetCommentsForComparedCommit", params, optFns, c.addOperationGetCommentsForComparedCommitMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*GetCommentsForComparedCommitOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type GetCommentsForComparedCommitInput struct {

	// To establish the directionality of the comparison, the full commit ID of the
	// after commit.
	//
	// This member is required.
	AfterCommitId *string

	// The name of the repository where you want to compare commits.
	//
	// This member is required.
	RepositoryName *string

	// To establish the directionality of the comparison, the full commit ID of the
	// before commit.
	BeforeCommitId *string

	// A non-zero, non-negative integer used to limit the number of returned results.
	// The default is 100 comments, but you can configure up to 500.
	MaxResults *int32

	// An enumeration token that when provided in a request, returns the next batch of
	// the results.
	NextToken *string

	noSmithyDocumentSerde
}

type GetCommentsForComparedCommitOutput struct {

	// A list of comment objects on the compared commit.
	CommentsForComparedCommitData []types.CommentsForComparedCommit

	// An enumeration token that can be used in a request to return the next batch of
	// the results.
	NextToken *string

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationGetCommentsForComparedCommitMiddlewares(stack *middleware.Stack, options Options) (err error) {
	err = stack.Serialize.Add(&awsAwsjson11_serializeOpGetCommentsForComparedCommit{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsAwsjson11_deserializeOpGetCommentsForComparedCommit{}, middleware.After)
	if err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddClientRequestIDMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddComputeContentLengthMiddleware(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = v4.AddComputePayloadSHA256Middleware(stack); err != nil {
		return err
	}
	if err = addRetryMiddlewares(stack, options); err != nil {
		return err
	}
	if err = addHTTPSignerV4Middleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addOpGetCommentsForComparedCommitValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opGetCommentsForComparedCommit(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	return nil
}

// GetCommentsForComparedCommitAPIClient is a client that implements the
// GetCommentsForComparedCommit operation.
type GetCommentsForComparedCommitAPIClient interface {
	GetCommentsForComparedCommit(context.Context, *GetCommentsForComparedCommitInput, ...func(*Options)) (*GetCommentsForComparedCommitOutput, error)
}

var _ GetCommentsForComparedCommitAPIClient = (*Client)(nil)

// GetCommentsForComparedCommitPaginatorOptions is the paginator options for
// GetCommentsForComparedCommit
type GetCommentsForComparedCommitPaginatorOptions struct {
	// A non-zero, non-negative integer used to limit the number of returned results.
	// The default is 100 comments, but you can configure up to 500.
	Limit int32

	// Set to true if pagination should stop if the service returns a pagination token
	// that matches the most recent token provided to the service.
	StopOnDuplicateToken bool
}

// GetCommentsForComparedCommitPaginator is a paginator for
// GetCommentsForComparedCommit
type GetCommentsForComparedCommitPaginator struct {
	options   GetCommentsForComparedCommitPaginatorOptions
	client    GetCommentsForComparedCommitAPIClient
	params    *GetCommentsForComparedCommitInput
	nextToken *string
	firstPage bool
}

// NewGetCommentsForComparedCommitPaginator returns a new
// GetCommentsForComparedCommitPaginator
func NewGetCommentsForComparedCommitPaginator(client GetCommentsForComparedCommitAPIClient, params *GetCommentsForComparedCommitInput, optFns ...func(*GetCommentsForComparedCommitPaginatorOptions)) *GetCommentsForComparedCommitPaginator {
	if params == nil {
		params = &GetCommentsForComparedCommitInput{}
	}

	options := GetCommentsForComparedCommitPaginatorOptions{}
	if params.MaxResults != nil {
		options.Limit = *params.MaxResults
	}

	for _, fn := range optFns {
		fn(&options)
	}

	return &GetCommentsForComparedCommitPaginator{
		options:   options,
		client:    client,
		params:    params,
		firstPage: true,
		nextToken: params.NextToken,
	}
}

// HasMorePages returns a boolean indicating whether more pages are available
func (p *GetCommentsForComparedCommitPaginator) HasMorePages() bool {
	return p.firstPage || (p.nextToken != nil && len(*p.nextToken) != 0)
}

// NextPage retrieves the next GetCommentsForComparedCommit page.
func (p *GetCommentsForComparedCommitPaginator) NextPage(ctx context.Context, optFns ...func(*Options)) (*GetCommentsForComparedCommitOutput, error) {
	if !p.HasMorePages() {
		return nil, fmt.Errorf("no more pages available")
	}

	params := *p.params
	params.NextToken = p.nextToken

	var limit *int32
	if p.options.Limit > 0 {
		limit = &p.options.Limit
	}
	params.MaxResults = limit

	result, err := p.client.GetCommentsForComparedCommit(ctx, &params, optFns...)
	if err != nil {
		return nil, err
	}
	p.firstPage = false

	prevToken := p.nextToken
	p.nextToken = result.NextToken

	if p.options.StopOnDuplicateToken &&
		prevToken != nil &&
		p.nextToken != nil &&
		*prevToken == *p.nextToken {
		p.nextToken = nil
	}

	return result, nil
}

func newServiceMetadataMiddleware_opGetCommentsForComparedCommit(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		SigningName:   "codecommit",
		OperationName: "GetCommentsForComparedCommit",
	}
}
