// Code generated by smithy-go-codegen DO NOT EDIT.

package omics

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/aws/signer/v4"
	"github.com/aws/aws-sdk-go-v2/service/omics/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
	"io"
)

// This operation uploads a specific part of a read set. If you upload a new part
// using a previously used part number, the previously uploaded part will be
// overwritten.
func (c *Client) UploadReadSetPart(ctx context.Context, params *UploadReadSetPartInput, optFns ...func(*Options)) (*UploadReadSetPartOutput, error) {
	if params == nil {
		params = &UploadReadSetPartInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "UploadReadSetPart", params, optFns, c.addOperationUploadReadSetPartMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*UploadReadSetPartOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type UploadReadSetPartInput struct {

	// The number of the part being uploaded.
	//
	// This member is required.
	PartNumber *int32

	// The source file for an upload part.
	//
	// This member is required.
	PartSource types.ReadSetPartSource

	// The read set data to upload for a part.
	//
	// This member is required.
	Payload io.Reader

	// The Sequence Store ID used for the multipart upload.
	//
	// This member is required.
	SequenceStoreId *string

	// The ID for the initiated multipart upload.
	//
	// This member is required.
	UploadId *string

	noSmithyDocumentSerde
}

type UploadReadSetPartOutput struct {

	// An identifier used to confirm that parts are being added to the intended upload.
	//
	// This member is required.
	Checksum *string

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationUploadReadSetPartMiddlewares(stack *middleware.Stack, options Options) (err error) {
	err = stack.Serialize.Add(&awsRestjson1_serializeOpUploadReadSetPart{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpUploadReadSetPart{}, middleware.After)
	if err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddClientRequestIDMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddComputeContentLengthMiddleware(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = v4.AddUnsignedPayloadMiddleware(stack); err != nil {
		return err
	}
	if err = v4.AddContentSHA256HeaderMiddleware(stack); err != nil {
		return err
	}
	if err = addRetryMiddlewares(stack, options); err != nil {
		return err
	}
	if err = addHTTPSignerV4Middleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addEndpointPrefix_opUploadReadSetPartMiddleware(stack); err != nil {
		return err
	}
	if err = addOpUploadReadSetPartValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opUploadReadSetPart(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	return nil
}

type endpointPrefix_opUploadReadSetPartMiddleware struct {
}

func (*endpointPrefix_opUploadReadSetPartMiddleware) ID() string {
	return "EndpointHostPrefix"
}

func (m *endpointPrefix_opUploadReadSetPartMiddleware) HandleSerialize(ctx context.Context, in middleware.SerializeInput, next middleware.SerializeHandler) (
	out middleware.SerializeOutput, metadata middleware.Metadata, err error,
) {
	if smithyhttp.GetHostnameImmutable(ctx) || smithyhttp.IsEndpointHostPrefixDisabled(ctx) {
		return next.HandleSerialize(ctx, in)
	}

	req, ok := in.Request.(*smithyhttp.Request)
	if !ok {
		return out, metadata, fmt.Errorf("unknown transport type %T", in.Request)
	}

	req.URL.Host = "storage-" + req.URL.Host

	return next.HandleSerialize(ctx, in)
}
func addEndpointPrefix_opUploadReadSetPartMiddleware(stack *middleware.Stack) error {
	return stack.Serialize.Insert(&endpointPrefix_opUploadReadSetPartMiddleware{}, `OperationSerializer`, middleware.After)
}

func newServiceMetadataMiddleware_opUploadReadSetPart(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		SigningName:   "omics",
		OperationName: "UploadReadSetPart",
	}
}
