// Code generated by smithy-go-codegen DO NOT EDIT.

package resiliencehub

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/aws/signer/v4"
	"github.com/aws/aws-sdk-go-v2/service/resiliencehub/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Lists all the Application Components in the Resilience Hub application.
func (c *Client) ListAppVersionAppComponents(ctx context.Context, params *ListAppVersionAppComponentsInput, optFns ...func(*Options)) (*ListAppVersionAppComponentsOutput, error) {
	if params == nil {
		params = &ListAppVersionAppComponentsInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "ListAppVersionAppComponents", params, optFns, c.addOperationListAppVersionAppComponentsMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*ListAppVersionAppComponentsOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type ListAppVersionAppComponentsInput struct {

	// The Amazon Resource Name (ARN) of the Resilience Hub application. The format
	// for this ARN is: arn: partition :resiliencehub: region : account :app/ app-id .
	// For more information about ARNs, see Amazon Resource Names (ARNs) (https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html)
	// in the AWS General Reference guide.
	//
	// This member is required.
	AppArn *string

	// The version of the Application Component.
	//
	// This member is required.
	AppVersion *string

	// Maximum number of Application Components to be displayed per Resilience Hub
	// application version.
	MaxResults *int32

	// Null, or the token from a previous call to get the next set of results.
	NextToken *string

	noSmithyDocumentSerde
}

type ListAppVersionAppComponentsOutput struct {

	// The Amazon Resource Name (ARN) of the Resilience Hub application. The format
	// for this ARN is: arn: partition :resiliencehub: region : account :app/ app-id .
	// For more information about ARNs, see Amazon Resource Names (ARNs) (https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html)
	// in the AWS General Reference guide.
	//
	// This member is required.
	AppArn *string

	// The Resilience Hub application version.
	//
	// This member is required.
	AppVersion *string

	// Defines an Application Component.
	AppComponents []types.AppComponent

	// The token for the next set of results, or null if there are no more results.
	NextToken *string

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationListAppVersionAppComponentsMiddlewares(stack *middleware.Stack, options Options) (err error) {
	err = stack.Serialize.Add(&awsRestjson1_serializeOpListAppVersionAppComponents{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpListAppVersionAppComponents{}, middleware.After)
	if err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddClientRequestIDMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddComputeContentLengthMiddleware(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = v4.AddComputePayloadSHA256Middleware(stack); err != nil {
		return err
	}
	if err = addRetryMiddlewares(stack, options); err != nil {
		return err
	}
	if err = addHTTPSignerV4Middleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addOpListAppVersionAppComponentsValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opListAppVersionAppComponents(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	return nil
}

// ListAppVersionAppComponentsAPIClient is a client that implements the
// ListAppVersionAppComponents operation.
type ListAppVersionAppComponentsAPIClient interface {
	ListAppVersionAppComponents(context.Context, *ListAppVersionAppComponentsInput, ...func(*Options)) (*ListAppVersionAppComponentsOutput, error)
}

var _ ListAppVersionAppComponentsAPIClient = (*Client)(nil)

// ListAppVersionAppComponentsPaginatorOptions is the paginator options for
// ListAppVersionAppComponents
type ListAppVersionAppComponentsPaginatorOptions struct {
	// Maximum number of Application Components to be displayed per Resilience Hub
	// application version.
	Limit int32

	// Set to true if pagination should stop if the service returns a pagination token
	// that matches the most recent token provided to the service.
	StopOnDuplicateToken bool
}

// ListAppVersionAppComponentsPaginator is a paginator for
// ListAppVersionAppComponents
type ListAppVersionAppComponentsPaginator struct {
	options   ListAppVersionAppComponentsPaginatorOptions
	client    ListAppVersionAppComponentsAPIClient
	params    *ListAppVersionAppComponentsInput
	nextToken *string
	firstPage bool
}

// NewListAppVersionAppComponentsPaginator returns a new
// ListAppVersionAppComponentsPaginator
func NewListAppVersionAppComponentsPaginator(client ListAppVersionAppComponentsAPIClient, params *ListAppVersionAppComponentsInput, optFns ...func(*ListAppVersionAppComponentsPaginatorOptions)) *ListAppVersionAppComponentsPaginator {
	if params == nil {
		params = &ListAppVersionAppComponentsInput{}
	}

	options := ListAppVersionAppComponentsPaginatorOptions{}
	if params.MaxResults != nil {
		options.Limit = *params.MaxResults
	}

	for _, fn := range optFns {
		fn(&options)
	}

	return &ListAppVersionAppComponentsPaginator{
		options:   options,
		client:    client,
		params:    params,
		firstPage: true,
		nextToken: params.NextToken,
	}
}

// HasMorePages returns a boolean indicating whether more pages are available
func (p *ListAppVersionAppComponentsPaginator) HasMorePages() bool {
	return p.firstPage || (p.nextToken != nil && len(*p.nextToken) != 0)
}

// NextPage retrieves the next ListAppVersionAppComponents page.
func (p *ListAppVersionAppComponentsPaginator) NextPage(ctx context.Context, optFns ...func(*Options)) (*ListAppVersionAppComponentsOutput, error) {
	if !p.HasMorePages() {
		return nil, fmt.Errorf("no more pages available")
	}

	params := *p.params
	params.NextToken = p.nextToken

	var limit *int32
	if p.options.Limit > 0 {
		limit = &p.options.Limit
	}
	params.MaxResults = limit

	result, err := p.client.ListAppVersionAppComponents(ctx, &params, optFns...)
	if err != nil {
		return nil, err
	}
	p.firstPage = false

	prevToken := p.nextToken
	p.nextToken = result.NextToken

	if p.options.StopOnDuplicateToken &&
		prevToken != nil &&
		p.nextToken != nil &&
		*prevToken == *p.nextToken {
		p.nextToken = nil
	}

	return result, nil
}

func newServiceMetadataMiddleware_opListAppVersionAppComponents(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		SigningName:   "resiliencehub",
		OperationName: "ListAppVersionAppComponents",
	}
}
