// Code generated by smithy-go-codegen DO NOT EDIT.

package servicecatalogappregistry

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/aws/signer/v4"
	"github.com/aws/aws-sdk-go-v2/service/servicecatalogappregistry/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Lists all of the resources that are associated with the specified application.
// Results are paginated. If you share an application, and a consumer account
// associates a tag query to the application, all of the users who can access the
// application can also view the tag values in all accounts that are associated
// with it using this API.
func (c *Client) ListAssociatedResources(ctx context.Context, params *ListAssociatedResourcesInput, optFns ...func(*Options)) (*ListAssociatedResourcesOutput, error) {
	if params == nil {
		params = &ListAssociatedResourcesInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "ListAssociatedResources", params, optFns, c.addOperationListAssociatedResourcesMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*ListAssociatedResourcesOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type ListAssociatedResourcesInput struct {

	// The name, ID, or ARN of the application.
	//
	// This member is required.
	Application *string

	// The upper bound of the number of results to return (cannot exceed 25). If this
	// parameter is omitted, it defaults to 25. This value is optional.
	MaxResults *int32

	// The token to use to get the next page of results after a previous API call.
	NextToken *string

	noSmithyDocumentSerde
}

type ListAssociatedResourcesOutput struct {

	// The token to use to get the next page of results after a previous API call.
	NextToken *string

	// Information about the resources.
	Resources []types.ResourceInfo

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationListAssociatedResourcesMiddlewares(stack *middleware.Stack, options Options) (err error) {
	err = stack.Serialize.Add(&awsRestjson1_serializeOpListAssociatedResources{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpListAssociatedResources{}, middleware.After)
	if err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddClientRequestIDMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddComputeContentLengthMiddleware(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = v4.AddComputePayloadSHA256Middleware(stack); err != nil {
		return err
	}
	if err = addRetryMiddlewares(stack, options); err != nil {
		return err
	}
	if err = addHTTPSignerV4Middleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addOpListAssociatedResourcesValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opListAssociatedResources(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	return nil
}

// ListAssociatedResourcesAPIClient is a client that implements the
// ListAssociatedResources operation.
type ListAssociatedResourcesAPIClient interface {
	ListAssociatedResources(context.Context, *ListAssociatedResourcesInput, ...func(*Options)) (*ListAssociatedResourcesOutput, error)
}

var _ ListAssociatedResourcesAPIClient = (*Client)(nil)

// ListAssociatedResourcesPaginatorOptions is the paginator options for
// ListAssociatedResources
type ListAssociatedResourcesPaginatorOptions struct {
	// The upper bound of the number of results to return (cannot exceed 25). If this
	// parameter is omitted, it defaults to 25. This value is optional.
	Limit int32

	// Set to true if pagination should stop if the service returns a pagination token
	// that matches the most recent token provided to the service.
	StopOnDuplicateToken bool
}

// ListAssociatedResourcesPaginator is a paginator for ListAssociatedResources
type ListAssociatedResourcesPaginator struct {
	options   ListAssociatedResourcesPaginatorOptions
	client    ListAssociatedResourcesAPIClient
	params    *ListAssociatedResourcesInput
	nextToken *string
	firstPage bool
}

// NewListAssociatedResourcesPaginator returns a new
// ListAssociatedResourcesPaginator
func NewListAssociatedResourcesPaginator(client ListAssociatedResourcesAPIClient, params *ListAssociatedResourcesInput, optFns ...func(*ListAssociatedResourcesPaginatorOptions)) *ListAssociatedResourcesPaginator {
	if params == nil {
		params = &ListAssociatedResourcesInput{}
	}

	options := ListAssociatedResourcesPaginatorOptions{}
	if params.MaxResults != nil {
		options.Limit = *params.MaxResults
	}

	for _, fn := range optFns {
		fn(&options)
	}

	return &ListAssociatedResourcesPaginator{
		options:   options,
		client:    client,
		params:    params,
		firstPage: true,
		nextToken: params.NextToken,
	}
}

// HasMorePages returns a boolean indicating whether more pages are available
func (p *ListAssociatedResourcesPaginator) HasMorePages() bool {
	return p.firstPage || (p.nextToken != nil && len(*p.nextToken) != 0)
}

// NextPage retrieves the next ListAssociatedResources page.
func (p *ListAssociatedResourcesPaginator) NextPage(ctx context.Context, optFns ...func(*Options)) (*ListAssociatedResourcesOutput, error) {
	if !p.HasMorePages() {
		return nil, fmt.Errorf("no more pages available")
	}

	params := *p.params
	params.NextToken = p.nextToken

	var limit *int32
	if p.options.Limit > 0 {
		limit = &p.options.Limit
	}
	params.MaxResults = limit

	result, err := p.client.ListAssociatedResources(ctx, &params, optFns...)
	if err != nil {
		return nil, err
	}
	p.firstPage = false

	prevToken := p.nextToken
	p.nextToken = result.NextToken

	if p.options.StopOnDuplicateToken &&
		prevToken != nil &&
		p.nextToken != nil &&
		*prevToken == *p.nextToken {
		p.nextToken = nil
	}

	return result, nil
}

func newServiceMetadataMiddleware_opListAssociatedResources(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		SigningName:   "servicecatalog",
		OperationName: "ListAssociatedResources",
	}
}
