// Code generated by smithy-go-codegen DO NOT EDIT.

package signer

import (
	"context"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/aws/signer/v4"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Signs a binary payload and returns a signature envelope.
func (c *Client) SignPayload(ctx context.Context, params *SignPayloadInput, optFns ...func(*Options)) (*SignPayloadOutput, error) {
	if params == nil {
		params = &SignPayloadInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "SignPayload", params, optFns, c.addOperationSignPayloadMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*SignPayloadOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type SignPayloadInput struct {

	// Specifies the object digest (hash) to sign.
	//
	// This member is required.
	Payload []byte

	// Payload content type
	//
	// This member is required.
	PayloadFormat *string

	// The name of the signing profile.
	//
	// This member is required.
	ProfileName *string

	// The AWS account ID of the profile owner.
	ProfileOwner *string

	noSmithyDocumentSerde
}

type SignPayloadOutput struct {

	// Unique identifier of the signing job.
	JobId *string

	// The AWS account ID of the job owner.
	JobOwner *string

	// Information including the signing profile ARN and the signing job ID. Clients
	// use metadata to signature records, for example, as annotations added to the
	// signature manifest inside an OCI registry.
	Metadata map[string]string

	// A cryptographic signature.
	Signature []byte

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationSignPayloadMiddlewares(stack *middleware.Stack, options Options) (err error) {
	err = stack.Serialize.Add(&awsRestjson1_serializeOpSignPayload{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpSignPayload{}, middleware.After)
	if err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddClientRequestIDMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddComputeContentLengthMiddleware(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = v4.AddComputePayloadSHA256Middleware(stack); err != nil {
		return err
	}
	if err = addRetryMiddlewares(stack, options); err != nil {
		return err
	}
	if err = addHTTPSignerV4Middleware(stack, options); err != nil {
		return err
	}
	if err = awsmiddleware.AddRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addOpSignPayloadValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opSignPayload(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = awsmiddleware.AddRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	return nil
}

func newServiceMetadataMiddleware_opSignPayload(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		SigningName:   "signer",
		OperationName: "SignPayload",
	}
}
